@@ -31,6 +31,7 @@
 #define SWITCHYARD_RTP_PORT 5004
 #define SWITCHYARD_SHM_KEY 0x59D2B831
 #define SWITCHYARD_SAVESOURCES_INTERVAL 5000
-
+#define SWITCHYARD_SETTINGS_ORGANIZATION "Paravel Systems"
+#define SWITCHYARD_SETTINGS_APPLICATION "Switchyard"
 
 #endif  // SYCONFIG_H
