@@ -6,29 +6,48 @@
 //     All Rights Reserved.
 //
 
-#include <arpa/inet.h>
 #include <errno.h>
-#include <ifaddrs.h>
-#include <net/if.h>
-#include <sys/ioctl.h>
 #include <stdio.h>
 #include <string.h>
-#include <sys/ipc.h>
-#include <sys/shm.h>
-#include <sys/socket.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
 
+#ifdef WIN32
+#include <Winsock2.h>
+#include <Ws2tcpip.h>
+#include <mswsock.h>
+#include <iphlpapi.h>
+#else
+#include <arpa/inet.h>
+#include <net/if.h>
+#include <sys/ioctl.h>
+#endif  // WIN32
+#ifdef OSX
+#include <ifaddrs.h>
+#endif  // OSX
+
 #include <QtCore/QDir>
+#include <QtCore/QSettings>
 #include <QtCore/QStringList>
 
-#include "sylogger.h"
+#include "sysyslog.h"
 #include "syprofile.h"
 #include "syrouting.h"
 
 SyRouting::SyRouting(unsigned d_slots,unsigned s_slots,unsigned gpis,unsigned gpos)
 {
+  //
+  // Initialize Networking
+  //
+#ifdef WIN32
+  WSADATA data;
+  if(WSAStartup(2,&data)!=0) {
+    SySyslog(LOG_ERR,"Winsock initialialization failed");
+    exit(256);
+  }
+#endif  // WIN32
+
   //
   // Create Database Directory
   //
@@ -87,10 +106,15 @@ QString SyRouting::hostName() const
 #ifdef OSX
   char hostname[sysconf(_SC_HOST_NAME_MAX)];
     gethostname(hostname,sysconf(_SC_HOST_NAME_MAX));
-#else
+#endif  // OSX
+#ifdef LINUX
   char hostname[HOST_NAME_MAX];
   gethostname(hostname,HOST_NAME_MAX);
-#endif  // OSX
+#endif  // LINUX
+#ifdef WIN32
+  char hostname[256];
+  gethostname(hostname,256);
+#endif  // WIN32
   return QString(hostname).split(".")[0];
 }
 
@@ -319,6 +343,15 @@ QString SyRouting::nicDevice(unsigned n)
 
 void SyRouting::subscribe(const QHostAddress &addr)
 {
+#ifdef WIN32
+  struct ip_mreq_source imr;
+
+  memset(&imr,0,sizeof(imr));
+  imr.imr_multiaddr.s_addr=htonl(addr.toIPv4Address());
+  imr.imr_interface.s_addr=nic_addr;
+  setsockopt(sy_subscription_socket,IPPROTO_IP,IP_ADD_MEMBERSHIP,
+	     (char *)&imr,sizeof(imr));
+#else
   struct ip_mreqn mreq;
 
   memset(&mreq,0,sizeof(mreq));
@@ -331,12 +364,23 @@ void SyRouting::subscribe(const QHostAddress &addr)
 #else
   setsockopt(sy_subscription_socket,SOL_IP,IP_ADD_MEMBERSHIP,
 	     &mreq,sizeof(mreq));
-#endif
+#endif  // OSX
+
+#endif // WIN32
 }
 
 
 void SyRouting::unsubscribe(const QHostAddress &addr)
 {
+#ifdef WIN32
+  struct ip_mreq_source imr;
+
+  memset(&imr,0,sizeof(imr));
+  imr.imr_multiaddr.s_addr=htonl(addr.toIPv4Address());
+  imr.imr_interface.s_addr=nic_addr;
+  setsockopt(sy_subscription_socket,IPPROTO_IP,IP_DROP_MEMBERSHIP,
+	     (char *)&imr,sizeof(imr));
+#else
   struct ip_mreqn mreq;
 
   memset(&mreq,0,sizeof(mreq));
@@ -348,6 +392,8 @@ void SyRouting::unsubscribe(const QHostAddress &addr)
 #else
   setsockopt(sy_subscription_socket,SOL_IP,IP_DROP_MEMBERSHIP,&mreq,sizeof(mreq));
 #endif  // OSX
+
+#endif  // WIN32
 }
 
 
@@ -365,6 +411,24 @@ int SyRouting::rtpSendSocket() const
 
 void SyRouting::load()
 {
+#ifdef WIN32
+  QSettings *s= new QSettings(QSettings::SystemScope,
+			      SWITCHYARD_SETTINGS_ORGANIZATION,
+			      SWITCHYARD_SETTINGS_APPLICATION);
+  setNicAddress(QHostAddress(s->value("NicAddress").toString()));
+  for(unsigned i=0;i<srcSlots();i++) {
+    QString key=QString().sprintf("Slot%u",i+1);
+    setSrcAddress(i,s->value(key+"/SourceAddress").toString());
+    setSrcName(i,s->value(key+"/SourceName").toString());
+    setSrcEnabled(i,s->value(key+"/SourceEnabled").toUInt());
+  }
+  for(unsigned i=0;i<dstSlots();i++) {
+    QString key=QString().sprintf("Slot%u",i+1);
+    setDstAddress(i,s->value(key+"/DestinationAddress").toString());
+    setDstName(i,s->value(key+"/DestinationName").toString());
+  }
+  delete s;
+#else
   QString section;
   SyProfile *p=new SyProfile();
   p->setSource(SWITCHYARD_ROUTING_FILE);
@@ -381,16 +445,36 @@ void SyRouting::load()
     setSrcEnabled(i,p->intValue(section,"SourceEnabled"));
   }
   for(unsigned i=0;i<dstSlots();i++) {
+    section=QString().sprintf("Slot%u",i+1);
     setDstAddress(i,p->addressValue(section,"DestinationAddress",""));
     setDstName(i,p->stringValue(section,"DestinationName",QString().sprintf("Destination %u",i+1)));
   }
-
   delete p;
+#endif  // WIN32
 }
 
 
 void SyRouting::save() const
 {
+#ifdef WIN32
+  QSettings *s= new QSettings(QSettings::SystemScope,
+			      SWITCHYARD_SETTINGS_ORGANIZATION,
+			      SWITCHYARD_SETTINGS_APPLICATION);
+  s->setValue("NicAddress",nicAddress().toString());
+  for(int i=0;i<SWITCHYARD_MAX_SLOTS;i++) {
+    QString key=QString().sprintf("Slot%u",i+1);
+    if(i<(int)srcSlots()) {
+      s->setValue(key+"/SourceAddress",srcAddress(i).toString());
+      s->setValue(key+"/SourceName",srcName(i));
+      s->setValue(key+"/SourceEnabled",srcEnabled(i));
+    }
+    if(i<(int)dstSlots()) {
+      s->setValue(key+"/DestinationAddress",dstAddress(i).toString());
+      s->setValue(key+"/DestinationName",dstName(i));
+    }
+  }
+  delete s;
+#else
   FILE *f=NULL;
   QString tempfile=QString(SWITCHYARD_ROUTING_FILE)+"-temp";
 
@@ -420,6 +504,7 @@ void SyRouting::save() const
   }
   fclose(f);
   rename(tempfile.toAscii(),SWITCHYARD_ROUTING_FILE);
+#endif  // WIN32
 }
 
 
@@ -448,6 +533,20 @@ QString SyRouting::dumpAddress(uint32_t addr)
 }
 
 
+QString SyRouting::socketErrorString(const QString &msg)
+{
+  QString ret=msg;
+#ifdef WIN32
+  int err=WSAGetLastError();
+  ret+=QString().sprintf(" [Winsock error: %d]",err);
+#else
+  ret+=QString(" [")+strerror(errno)+"]";
+#endif  // WIN32
+
+  return ret;
+}
+
+
 int SyRouting::GetSlotByGpio(int gpio) const
 {
   for(unsigned i=0;i<dstSlots();i++) {
@@ -463,17 +562,15 @@ int SyRouting::GetSlotByGpio(int gpio) const
 void SyRouting::LoadInterfaces()
 {
   if((sy_subscription_socket=socket(PF_INET,SOCK_DGRAM,IPPROTO_IP))<0) {
-    SySyslog(LOG_ERR,QString().
-	     sprintf("unable to create RTP subscription socket [%s]",
-		     strerror(errno)));
+    SySyslog(LOG_ERR,
+      SyRouting::socketErrorString("unable to create RTP suscription socket"));
     exit(256);
   }
   if((sy_rtp_send_socket=socket(PF_INET,SOCK_DGRAM,IPPROTO_IP))<0) {
-    SySyslog(LOG_ERR,QString().
-	     sprintf("unable to creat rtp send socket [%s]",
-		     strerror(errno)));
+    SySyslog(LOG_ERR,
+      SyRouting::socketErrorString("unable to create RTP send socket"));
     exit(256);
-  }
+  } 
 
 #ifdef OSX
   struct ifaddrs *ifap=NULL;
@@ -507,14 +604,8 @@ void SyRouting::LoadInterfaces()
     }
   } while((ifa=ifa->ifa_next)!=NULL);
   freeifaddrs(ifap);
-  /*
-  for(unsigned i=0;i<sy_nic_addresses.size();i++) {
-    printf("%s: %s  %s\n",(const char *)sy_nic_devices[i].toAscii(),
-	   (const char *)sy_nic_addresses[i].toString().toAscii(),
-	   (const char *)sy_nic_netmasks[i].toString().toAscii());
-  }
-  */
-#else
+#endif  // OSX
+#ifdef LINUX
   struct ifreq ifr;
   int index=0;
   uint64_t mac;
@@ -560,5 +651,47 @@ void SyRouting::LoadInterfaces()
     }
     ifr.ifr_ifindex=++index;
   }
-#endif  // OSX
+#endif  // LINUX
+#ifdef WIN32
+  IP_ADAPTER_INFO *info;
+  ULONG buf_len;
+  DWORD ret=0;
+
+  info=(IP_ADAPTER_INFO *)malloc(sizeof(IP_ADAPTER_INFO));
+  buf_len=sizeof(IP_ADAPTER_INFO);
+  if(GetAdaptersInfo(info,&buf_len)!=ERROR_SUCCESS) {
+    free(info);
+    info=(IP_ADAPTER_INFO *)malloc(buf_len);
+  }
+  if(GetAdaptersInfo(info,&buf_len)!=ERROR_SUCCESS) {
+    SySyslog(LOG_ERR,QString().sprintf("GetAdaptersInfo failed, error=%lu",ret));
+    exit(256);
+  }
+  PIP_ADAPTER_INFO a=info;
+  int count=0;
+  while(a) {
+    //printf("Name: %s\n",a->AdapterName);
+    //printf("Desc: %s\n",a->Description);
+    QString dev=QString().sprintf("%d: ",count+1);
+    for(unsigned i=0;i<a->AddressLength;i++) {
+      dev+=QString().sprintf("%02X:",(int)a->Address[i]);
+    }
+    dev=dev.left(dev.length()-1);
+    sy_nic_devices.push_back(dev);
+    sy_nic_addresses.push_back(QHostAddress(a->IpAddressList.IpAddress.String));
+    sy_nic_netmasks.push_back(QHostAddress(a->IpAddressList.IpMask.String));
+    count++;
+    a=info->Next;
+  }
+  if(info!=NULL) {
+    free(info);
+  }
+#endif  // WIN32
+  /*
+  for(unsigned i=0;i<sy_nic_addresses.size();i++) {
+    printf("%s: %s  %s\n",(const char *)sy_nic_devices[i].toAscii(),
+	   (const char *)sy_nic_addresses[i].toString().toAscii(),
+	   (const char *)sy_nic_netmasks[i].toString().toAscii());
+  }
+  */
 }
