@@ -10,9 +10,13 @@
 #ifndef SYRTP_SERVER_H
 #define SYRTP_SERVER_H
 
+#ifdef WIN32
+#include <Winsock2.h>
+#else
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netinet/ip.h>
+#endif  // WIN32
 #include <pthread.h>
 
 #include <QtCore/QObject>
@@ -24,7 +28,7 @@ class SyRtpServer : public QObject
 {
   Q_OBJECT;
  public:
-  SyRtpServer(void *(*callback_func)(unsigned,const char *,int,SyRouting *,void *),
+  SyRtpServer(void *(*callback_func)(uint32_t,const char *,int,SyRouting *,void *),
 	      void *callback_priv,SyRouting *routing,QObject *parent=0);
   ~SyRtpServer();
   void shutdown();
