@@ -16,20 +16,20 @@
 #include <QtCore/QTimer>
 
 #include <sy/sycmdswitch.h>
-#include <sy/sylogger.h>
+#include <sy/sysyslog.h>
 
 #include "syd.h"
 
 bool global_exiting=false;
 
-void *RtpCallback(unsigned dst_slot,const char *data,int len,
+void *RtpCallback(uint32_t dst_addr,const char *data,int len,
 		  SyRouting *r,void *priv)
 {
   static unsigned i;
 
   for(i=0;i<r->src_slots;i++) {
     if(r->src_enabled[i]&&(r->src_addr[i]!=0)&&
-       (dst_slot==r->dst_addr[i])) {
+       (dst_addr==r->dst_addr[i])) {
       r->writeRtpData(i,data,len);
     }
   }
@@ -52,7 +52,6 @@ MainObject::MainObject(QObject *parent)
   : QObject(parent)
 {
   bool debug=false;
-  FILE *f=NULL;
 
   //
   // Process Command Line
@@ -72,7 +71,7 @@ MainObject::MainObject(QObject *parent)
   //
   // Open Logger
   //
-  SyOpenLog("syd");
+  SyOpenLog("syd",LOG_PERROR,LOG_DAEMON);
 
   //
   // Load Routing Rules
@@ -87,7 +86,7 @@ MainObject::MainObject(QObject *parent)
   //
   // Start Advertisements
   //
-  syd_adv=new SyAdvServer(syd_routing,true,this);
+  syd_adv=new SyAdvServer(syd_routing,false,this);
 
   //
   // Start GPIO Processing
@@ -101,6 +100,8 @@ MainObject::MainObject(QObject *parent)
   //
   // Detach and write PID file
   //
+#ifdef LINUX
+  FILE *f=NULL;
   if(!debug) {
     daemon(0,0);
     if((f=fopen(SYD_PID_FILE,"w"))!=NULL) {
@@ -108,6 +109,7 @@ MainObject::MainObject(QObject *parent)
       fclose(f);
     }
   }
+#endif  // LINUX
 
   //
   // Start RTP
@@ -169,7 +171,7 @@ void MainObject::exitTimerData()
 void MainObject::exitData()
 {
   SySyslog(LOG_DEBUG,"calling exit handler");
-  unlink(SWITCHYARD_SOURCES_FILE);
+  delete syd_adv;
   unlink(SYD_PID_FILE);
   exit(0);
 }
